@@ -2,12 +2,12 @@ package com.unciv.logic
 
 import com.unciv.Constants
 import com.unciv.logic.city.CityConstructions
-import com.unciv.logic.city.PerpetualConstruction
 import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.civilization.diplomacy.DiplomacyManager
 import com.unciv.logic.civilization.managers.TechManager
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.ModOptions
+import com.unciv.models.ruleset.PerpetualConstruction
 import com.unciv.models.ruleset.Ruleset
 
 /**
@@ -27,6 +27,17 @@ object BackwardCompatibility {
     fun GameInfo.removeMissingModReferences() {
         tileMap.removeMissingTerrainModReferences(ruleset)
 
+        removeUnitsAndPromotions()
+
+        // Mod decided you can't repair things anymore - get rid of old pillaged improvements
+        removeOldPillagedImprovements()
+
+        handleMissingReferencesForEachCity()
+
+        removeTechAndPolicies()
+    }
+
+    private fun GameInfo.removeUnitsAndPromotions() {
         for (tile in tileMap.values) {
             for (unit in tile.getUnits()) {
                 if (!ruleset.units.containsKey(unit.name)) tile.removeUnit(unit)
@@ -36,8 +47,9 @@ object BackwardCompatibility {
                         unit.promotions.promotions.remove(promotion)
             }
         }
+    }
 
-        // Mod decided you can't repair things anymore - get rid of old pillaged improvements
+    private fun GameInfo.removeOldPillagedImprovements() {
         if (!ruleset.tileImprovements.containsKey(Constants.repair))
             for (tile in tileMap.values) {
                 if (tile.roadIsPillaged) {
@@ -49,22 +61,20 @@ object BackwardCompatibility {
                     tile.improvementIsPillaged = false
                 }
             }
+    }
 
-
+    private fun GameInfo.handleMissingReferencesForEachCity() {
         for (city in civilizations.asSequence().flatMap { it.cities.asSequence() }) {
 
-            changeBuildingNameIfNotInRuleset(ruleset, city.cityConstructions, "Hanse", "Bank")
-
-            for (building in city.cityConstructions.builtBuildings.toHashSet()) {
-
-                if (!ruleset.buildings.containsKey(building))
-                    city.cityConstructions.builtBuildings.remove(building)
+            for (building in city.cityConstructions.getBuiltBuildings()) {
+                if (!ruleset.buildings.containsKey(building.name))
+                    city.cityConstructions.removeBuilding(building)
             }
 
             fun isInvalidConstruction(construction: String) =
                 !ruleset.buildings.containsKey(construction)
-                        && !ruleset.units.containsKey(construction)
-                        && !PerpetualConstruction.perpetualConstructionsMap.containsKey(construction)
+                    && !ruleset.units.containsKey(construction)
+                    && !PerpetualConstruction.perpetualConstructionsMap.containsKey(construction)
 
             // Remove invalid buildings or units from the queue - don't just check buildings and units because it might be a special construction as well
             for (construction in city.cityConstructions.constructionQueue.toList()) {
@@ -76,7 +86,9 @@ object BackwardCompatibility {
                 if (isInvalidConstruction(construction))
                     city.cityConstructions.inProgressConstructions.remove(construction)
         }
+    }
 
+    private fun GameInfo.removeTechAndPolicies() {
         for (civInfo in civilizations) {
             for (tech in civInfo.tech.techsResearched.toList())
                 if (!ruleset.technologies.containsKey(tech))
@@ -100,12 +112,12 @@ object BackwardCompatibility {
         if (ruleSet.buildings.containsKey(oldBuildingName))
             return
         // Replace in built buildings
-        if (cityConstructions.builtBuildings.contains(oldBuildingName)) {
-            cityConstructions.builtBuildings.remove(oldBuildingName)
-            cityConstructions.builtBuildings.add(newBuildingName)
+        if (cityConstructions.isBuilt(oldBuildingName)) {
+            cityConstructions.removeBuilding(oldBuildingName)
+            cityConstructions.addBuilding(newBuildingName)
         }
         // Replace in construction queue
-        if (!cityConstructions.builtBuildings.contains(newBuildingName) && !cityConstructions.constructionQueue.contains(newBuildingName))
+        if (!cityConstructions.isBuilt(newBuildingName) && !cityConstructions.constructionQueue.contains(newBuildingName))
             cityConstructions.constructionQueue = cityConstructions.constructionQueue
                 .map { if (it == oldBuildingName) newBuildingName else it }
                 .toMutableList()
@@ -113,7 +125,7 @@ object BackwardCompatibility {
             cityConstructions.constructionQueue.remove(oldBuildingName)
         // Replace in in-progress constructions
         if (cityConstructions.inProgressConstructions.containsKey(oldBuildingName)) {
-            if (!cityConstructions.builtBuildings.contains(newBuildingName) && !cityConstructions.inProgressConstructions.containsKey(newBuildingName))
+            if (!cityConstructions.isBuilt(newBuildingName) && !cityConstructions.inProgressConstructions.containsKey(newBuildingName))
                 cityConstructions.inProgressConstructions[newBuildingName] = cityConstructions.inProgressConstructions[oldBuildingName]!!
             cityConstructions.inProgressConstructions.remove(oldBuildingName)
         }
@@ -156,10 +168,8 @@ object BackwardCompatibility {
     }
 
     /** Move max XP from barbarians to new home */
-    @Suppress("DEPRECATION")
-    fun ModOptions.updateDeprecations() {
-    }
-
+    @Suppress("DEPRECATION", "EmptyFunctionBlock")
+    fun ModOptions.updateDeprecations() { }
 
     /** Convert from Fortify X to Fortify and save off X */
     fun GameInfo.convertFortify() {
@@ -175,18 +185,6 @@ object BackwardCompatibility {
         }
     }
 
-    private fun isOldFormat(manager: BarbarianManager): Boolean {
-        val keys = manager.camps.keys as Set<Any>
-        val iterator = keys.iterator()
-        while (iterator.hasNext()) {
-            val key = iterator.next()
-            if (key is String) {
-                return true
-            }
-        }
-        return false
-    }
-
     fun GameInfo.migrateToTileHistory() {
         if (historyStartTurn >= 0) return
         for (tile in getCities().flatMap { it.getTiles() }) {
