@@ -1,6 +1,7 @@
 package com.unciv.logic.automation.unit
 
 import com.unciv.Constants
+import com.unciv.UncivGame
 import com.unciv.logic.automation.Automation
 import com.unciv.logic.automation.civilization.NextTurnAutomation
 import com.unciv.logic.battle.Battle
@@ -8,6 +9,7 @@ import com.unciv.logic.battle.BattleDamage
 import com.unciv.logic.battle.CityCombatant
 import com.unciv.logic.battle.ICombatant
 import com.unciv.logic.battle.MapUnitCombatant
+import com.unciv.logic.battle.TargetHelper
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.NotificationCategory
@@ -109,7 +111,6 @@ object UnitAutomation {
                 && unit.movement.canReach(tile) // expensive, evaluate last
     }
 
-    @JvmStatic
     fun wander(unit: MapUnit, stayInTerritory: Boolean = false, tilesToAvoid:Set<Tile> = setOf()) {
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         val reachableTiles = unitDistanceToTiles
@@ -128,6 +129,8 @@ object UnitAutomation {
     }
 
     internal fun tryUpgradeUnit(unit: MapUnit): Boolean {
+        val isHuman = unit.civ.isHuman()
+        if (!UncivGame.Current.settings.automatedUnitsCanUpgrade && isHuman) return false
         if (unit.baseUnit.upgradesTo == null) return false
         val upgradedUnit = unit.upgrade.getUnitToUpgradeTo()
         if (!upgradedUnit.isBuildable(unit.civ)) return false // for resource reasons, usually
@@ -156,8 +159,11 @@ object UnitAutomation {
             return
         }
 
-        if (unit.promotions.canBePromoted()) {
+        if (unit.promotions.canBePromoted() &&
+            // Restrict Human automated units from promotions via setting
+                (UncivGame.Current.settings.automatedUnitsChoosePromotions || unit.civ.isAI())) {
             val availablePromotions = unit.promotions.getAvailablePromotions()
+                .filterNot { it.hasUnique(UniqueType.SkipPromotion) }
             if (availablePromotions.any())
                 unit.promotions.addPromotion(availablePromotions.toList().random().name)
         }
@@ -173,7 +179,7 @@ object UnitAutomation {
         if (unit.baseUnit.isAirUnit() && unit.canIntercept())
             return SpecificUnitAutomation.automateFighter(unit)
 
-        if (unit.baseUnit.isAirUnit())
+        if (unit.baseUnit.isAirUnit() && !unit.baseUnit.isNuclearWeapon())
             return SpecificUnitAutomation.automateBomber(unit)
 
         if (unit.baseUnit.isNuclearWeapon())
@@ -204,7 +210,9 @@ object UnitAutomation {
         // Focus all units without a specific target on the enemy city closest to one of our cities
         if (tryHeadTowardsEnemyCity(unit)) return
 
-        if (tryGarrisoningUnit(unit)) return
+        if (tryGarrisoningRangedLandUnit(unit)) return
+
+        if (tryStationingMeleeNavalUnit(unit)) return
 
         if (unit.health < 80 && tryHealUnit(unit)) return
 
@@ -465,11 +473,6 @@ object UnitAutomation {
         return unit.currentMovement == 0f
     }
 
-    /** Get a list of visible tiles which have something attackable */
-    fun getBombardableTiles(city: City): Sequence<Tile> =
-            city.getCenterTile().getTilesInDistance(city.range)
-                    .filter { it.isVisible(city.civ) && BattleHelper.containsAttackableEnemy(it, CityCombatant(city)) }
-
     /** Move towards the closest attackable enemy of the [unit].
      *
      *  Limited by [CLOSE_ENEMY_TURNS_AWAY_LIMIT] and [CLOSE_ENEMY_TILES_AWAY_LIMIT].
@@ -480,7 +483,7 @@ object UnitAutomation {
                 unit.getTile().position,
                 unit.getMaxMovement() * CLOSE_ENEMY_TURNS_AWAY_LIMIT
         )
-        var closeEnemies = BattleHelper.getAttackableEnemies(
+        var closeEnemies = TargetHelper.getAttackableEnemies(
             unit,
             unitDistanceToTiles,
             tilesToCheck = unit.getTile().getTilesInDistance(CLOSE_ENEMY_TILES_AWAY_LIMIT).toList()
@@ -631,7 +634,7 @@ object UnitAutomation {
             .sum() // City heals 20 per turn
 
         if (expectedDamagePerTurn < city.health && // If we can take immediately, go for it
-                (expectedDamagePerTurn <= 20 || city.health / (expectedDamagePerTurn-20) > 5)){ // otherwise check if we can take within a couple of turns
+            (expectedDamagePerTurn <= 20 || city.health / (expectedDamagePerTurn-20) > 5)){ // otherwise check if we can take within a couple of turns
 
             // We won't be able to take this even with 5 turns of continuous damage!
             // don't head straight to the city, try to head to landing grounds -
@@ -672,7 +675,7 @@ object UnitAutomation {
     }
 
     private fun chooseBombardTarget(city: City): ICombatant? {
-        var targets = getBombardableTiles(city).map { Battle.getMapCombatantOfTile(it)!! }
+        var targets = TargetHelper.getBombardableTiles(city).map { Battle.getMapCombatantOfTile(it)!! }
         if (targets.none()) return null
 
         val siegeUnits = targets
@@ -726,7 +729,7 @@ object UnitAutomation {
 
     }
 
-    private fun tryGarrisoningUnit(unit: MapUnit): Boolean {
+    private fun tryGarrisoningRangedLandUnit(unit: MapUnit): Boolean {
         if (unit.baseUnit.isMelee() || unit.baseUnit.isWaterUnit()) return false // don't garrison melee units, they're not that good at it
         val citiesWithoutGarrison = unit.civ.cities.filter {
             val centerTile = it.getCenterTile()
@@ -749,8 +752,10 @@ object UnitAutomation {
         } else {
             if (unit.getTile().isCityCenter() &&
                     isCityThatNeedsDefendingInWartime(unit.getTile().getCity()!!)) return true
-            citiesWithoutGarrison.asSequence()
+            val citiesWithoutGarrisonThatNeedDefending = citiesWithoutGarrison.asSequence()
                     .filter { isCityThatNeedsDefendingInWartime(it) }
+            if (citiesWithoutGarrisonThatNeedDefending.any()) citiesWithoutGarrisonThatNeedDefending
+            else citiesWithoutGarrison.asSequence()
         }
 
         val closestReachableCityNeedsDefending = citiesToTry
@@ -761,6 +766,35 @@ object UnitAutomation {
         return true
     }
 
+    private fun tryStationingMeleeNavalUnit(unit: MapUnit): Boolean {
+        fun isMeleeNaval(mapUnit: MapUnit) = mapUnit.baseUnit.isMelee() && mapUnit.type.isWaterUnit()
+
+        if (!isMeleeNaval(unit)) return false
+        val closeCity = unit.getTile().getTilesInDistance(3)
+            .firstOrNull { it.isCityCenter() }
+
+        // We're the closest unit to this city, we should stay here :)
+        if (closeCity != null && closeCity.getTilesInDistance(3)
+                .flatMap { it.getUnits() }
+                .firstOrNull {isMeleeNaval(it)} == unit
+            && unit.movement.canReach(closeCity)) {
+            unit.movement.headTowards(closeCity)
+            return true
+        }
+
+        val citiesWithoutNavalDefence = unit.civ.cities.filter { it.isCoastal() }
+            .filter { it.getCenterTile().aerialDistanceTo(unit.getTile()) < 20 } // Not too far away
+            .filter { it.getCenterTile().getTilesInDistance(3)
+                .flatMap { it.getUnits() }
+                .none { isMeleeNaval(it) }}
+
+        val reachableCity = citiesWithoutNavalDefence.firstOrNull {
+            unit.movement.canReach(it.getCenterTile())
+        } ?: return false
+        unit.movement.headTowards(reachableCity.getCenterTile())
+        return true
+    }
+
     /** This is what a unit with the 'explore' action does.
     It also explores, but also has other functions, like healing if necessary. */
     fun automatedExplore(unit: MapUnit) {
