@@ -19,6 +19,7 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unique.endTurn
 import com.unciv.models.stats.Stats
 import com.unciv.ui.components.MayaCalendar
+import com.unciv.ui.screens.worldscreen.status.NextTurnProgress
 import com.unciv.utils.Log
 import kotlin.math.max
 import kotlin.math.min
@@ -27,7 +28,7 @@ import kotlin.random.Random
 class TurnManager(val civInfo: Civilization) {
 
 
-    fun startTurn() {
+    fun startTurn(progressBar: NextTurnProgress? = null) {
         if (civInfo.isSpectator()) return
 
         if (civInfo.isMajorCiv() && civInfo.isAlive()) {
@@ -66,7 +67,10 @@ class TurnManager(val civInfo: Civilization) {
         civInfo.cache.updateCitiesConnectedToCapital()
         startTurnFlags()
         updateRevolts()
-        for (city in civInfo.cities) CityTurnManager(city).startTurn()  // Most expensive part of startTurn
+        for (city in civInfo.cities) {
+            progressBar?.increment()
+            CityTurnManager(city).startTurn()  // Most expensive part of startTurn
+        }
 
         for (unit in civInfo.units.getCivUnits()) UnitTurnManager(unit).startTurn()
 
@@ -187,7 +191,7 @@ class TurnManager(val civInfo: Civilization) {
         repeat(rebelCount) {
             civInfo.gameInfo.tileMap.placeUnitNearTile(
                 spawnTile.position,
-                unitToSpawn.name,
+                unitToSpawn,
                 barbarians
             )
         }
@@ -218,7 +222,7 @@ class TurnManager(val civInfo: Civilization) {
             ((4 + Random.Default.nextInt(3)) * max(civInfo.gameInfo.speed.modifier, 1f)).toInt()
 
 
-    fun endTurn() {
+    fun endTurn(progressBar: NextTurnProgress? = null) {
         val notificationsLog = civInfo.notificationsLog
         val notificationsThisTurn = Civilization.NotificationsLog(civInfo.gameInfo.turns)
         notificationsThisTurn.notifications.addAll(civInfo.notifications)
@@ -278,8 +282,10 @@ class TurnManager(val civInfo: Civilization) {
 
         // To handle tile's owner issue (#8246), we need to run cities being razed first.
         // a city can be removed while iterating (if it's being razed) so we need to iterate over a copy - sorting does one
-        for (city in civInfo.cities.sortedByDescending { it.isBeingRazed })
+        for (city in civInfo.cities.sortedByDescending { it.isBeingRazed }) {
+            progressBar?.increment()
             CityTurnManager(city).endTurn()
+        }
 
         civInfo.temporaryUniques.endTurn()
 
@@ -301,8 +307,13 @@ class TurnManager(val civInfo: Civilization) {
             civInfo.gameInfo.victoryData =
                     VictoryData(civInfo.civName, victoryType, civInfo.gameInfo.turns)
 
-            for (civInfo in civInfo.gameInfo.civilizations)
-                civInfo.popupAlerts.add(PopupAlert(AlertType.GameHasBeenWon, civInfo.civName))
+            // Notify other human players about this civInfo's victory
+            for (otherCiv in civInfo.gameInfo.civilizations) {
+                // Skip winner, displaying VictoryScreen is handled separately in WorldScreen.update
+                // by checking `viewingCiv.isDefeated() || gameInfo.checkForVictory()`
+                if (otherCiv.playerType != PlayerType.Human || otherCiv == civInfo) continue
+                otherCiv.popupAlerts.add(PopupAlert(AlertType.GameHasBeenWon, ""))
+            }
         }
     }
 
