@@ -12,16 +12,21 @@ import com.unciv.models.metadata.Player
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.nation.Nation
 import com.unciv.models.ruleset.tile.TerrainType
+import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.unit.BaseUnit
 import java.lang.Integer.max
+import java.util.concurrent.ConcurrentHashMap
 import kotlin.math.abs
 
 /** An Unciv map with all properties as produced by the [map editor][com.unciv.ui.screens.mapeditorscreen.MapEditorScreen]
  * or [MapGenerator][com.unciv.logic.map.mapgenerator.MapGenerator]; or as part of a running [game][GameInfo].
  *
  * Note: Will be Serialized -> Take special care with lateinit and lazy!
+ *
+ * @param initialCapacity Passed to constructor of [tileList]
  */
-class TileMap : IsPartOfGameInfoSerialization {
+class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     companion object {
         /** Legacy way to store starting locations - now this is used only in [translateStartingLocationsFromMap] */
         const val startingLocationPrefix = "StartingLocation "
@@ -38,7 +43,7 @@ class TileMap : IsPartOfGameInfoSerialization {
 
     var mapParameters = MapParameters()
 
-    var tileList = ArrayList<Tile>()
+    var tileList = ArrayList<Tile>(initialCapacity)
 
     /** Structure geared for simple serialization by Gdx.Json (which is a little blind to kotlin collections, especially HashSet)
      * @param position [Vector2] of the location
@@ -58,6 +63,9 @@ class TileMap : IsPartOfGameInfoSerialization {
     @Transient
     var ruleset: Ruleset? = null
 
+    @Transient
+    var tileUniqueMapCache = ConcurrentHashMap<List<String>, UniqueMap>()
+
     @Transient
     var tileMatrix = ArrayList<ArrayList<Tile?>>() // this works several times faster than a hashmap, the performance difference is really astounding
 
@@ -93,11 +101,9 @@ class TileMap : IsPartOfGameInfoSerialization {
     //endregion
     //region Constructors
 
-    /** for json parsing, we need to have a default constructor */
-    constructor()
-
     /** creates a hexagonal map of given radius (filled with grassland) */
-    constructor(radius: Int, ruleset: Ruleset, worldWrap: Boolean = false) {
+    constructor(radius: Int, ruleset: Ruleset, worldWrap: Boolean = false)
+            : this (HexMath.getNumberOfTilesInHexagon(radius)) {
         startingLocations.clear()
         val firstAvailableLandTerrain = MapLandmassGenerator.getInitializationTerrain(ruleset, TerrainType.Land)
         for (vector in HexMath.getVectorsInDistance(Vector2.Zero, radius, worldWrap))
@@ -106,7 +112,8 @@ class TileMap : IsPartOfGameInfoSerialization {
     }
 
     /** creates a rectangular map of given width and height (filled with grassland) */
-    constructor(width: Int, height: Int, ruleset: Ruleset, worldWrap: Boolean = false) {
+    constructor(width: Int, height: Int, ruleset: Ruleset, worldWrap: Boolean = false)
+            : this(width * height) {
         startingLocations.clear()
         val firstAvailableLandTerrain = MapLandmassGenerator.getInitializationTerrain(ruleset, TerrainType.Land)
 
@@ -130,13 +137,17 @@ class TileMap : IsPartOfGameInfoSerialization {
 
     /** @return a deep-copy clone of the serializable fields, no transients initialized */
     fun clone(): TileMap {
-        val toReturn = TileMap()
-        toReturn.tileList.addAll(tileList.map { it.clone() })
+        val toReturn = TileMap(tileList.size)
+        toReturn.tileList.addAll(tileList.asSequence().map { it.clone() })
         toReturn.mapParameters = mapParameters
         toReturn.ruleset = ruleset
+
+        // Note during normal play this is empty. Supported for MapEditorScreen.getMapCloneForSave.
         toReturn.startingLocations.clear()
         toReturn.startingLocations.ensureCapacity(startingLocations.size)
         toReturn.startingLocations.addAll(startingLocations)
+        toReturn.tileUniqueMapCache = tileUniqueMapCache
+
         return toReturn
     }
 
@@ -383,7 +394,7 @@ class TileMap : IsPartOfGameInfoSerialization {
         // All the rest is to find missing nations
         try { // This can fail if the map contains a resource that isn't in the ruleset, in Tile.tileResource
             setTransients(ruleset)
-        } catch (ex: Exception) {
+        } catch (_: Exception) {
             return rulesetIncompatibilities
         }
         setStartingLocationsTransients()
@@ -411,7 +422,7 @@ class TileMap : IsPartOfGameInfoSerialization {
      */
     fun setTransients(ruleset: Ruleset? = null, setUnitCivTransients: Boolean = true) {
         if (ruleset != null) this.ruleset = ruleset
-        if (this.ruleset == null) throw(IllegalStateException("TileMap.setTransients called without ruleset"))
+        check(this.ruleset != null) { "TileMap.setTransients called without ruleset" }
 
         if (tileMatrix.isEmpty()) {
             val topY = tileList.asSequence().map { it.position.y.toInt() }.maxOrNull()!!
@@ -430,8 +441,9 @@ class TileMap : IsPartOfGameInfoSerialization {
         } else {
             // Yes the map generator calls this repeatedly, and we don't want to end up with an oversized tileMatrix
             // rightX is -leftX or -leftX + 1 or -leftX + 2
-            if (tileMatrix.size !in (1 - 2 * leftX)..(3 - 2 * leftX))
-                throw(IllegalStateException("TileMap.setTransients called on existing tileMatrix of different size"))
+            check(tileMatrix.size in (1 - 2 * leftX)..(3 - 2 * leftX)) {
+                "TileMap.setTransients called on existing tileMatrix of different size"
+            }
         }
 
         for (tileInfo in values) {
@@ -459,16 +471,13 @@ class TileMap : IsPartOfGameInfoSerialization {
     }
 
     fun removeMissingTerrainModReferences(ruleSet: Ruleset) {
+        // This will run before setTransients, so do not rely e.g. on Tile.ruleset being available.
+        // That rules out Tile.removeTerrainFeature, which refreshes object/unique caches
         for (tile in this.values) {
-            for (terrainFeature in tile.terrainFeatures.filter { !ruleSet.terrains.containsKey(it) })
-                tile.removeTerrainFeature(terrainFeature)
-            if (tile.resource != null && !ruleSet.tileResources.containsKey(tile.resource!!))
-                tile.resource = null
-            if (tile.improvement != null && !ruleSet.tileImprovements.containsKey(tile.improvement!!))
-                tile.changeImprovement(null)
+            tile.removeMissingTerrainModReferences(ruleSet)
         }
         for (startingLocation in startingLocations.toList())
-            if (startingLocation.nation !in ruleSet.nations.keys)
+            if (startingLocation.nation !in ruleSet.nations)
                 startingLocations.remove(startingLocation)
     }
 
@@ -478,12 +487,27 @@ class TileMap : IsPartOfGameInfoSerialization {
      * @param civInfo civilization to assign unit to
      * @return created [MapUnit] or null if no suitable location was found
      * */
+    fun placeUnitNearTile(
+        position: Vector2,
+        unitName: String,
+        civInfo: Civilization
+    ): MapUnit? {
+        val unit = gameInfo.ruleset.units[unitName]!!
+        return placeUnitNearTile(position, unit, civInfo)
+    }
+
+    /** Tries to place the [baseUnit] into the [Tile] closest to the given [position]
+     * @param position where to try to place the unit (or close - max 10 tiles distance)
+     * @param baseUnit [BaseUnit][com.unciv.models.ruleset.unit.BaseUnit] to create and place
+     * @param civInfo civilization to assign unit to
+     * @return created [MapUnit] or null if no suitable location was found
+     * */
     fun placeUnitNearTile(
             position: Vector2,
-            unitName: String,
+            baseUnit: BaseUnit,
             civInfo: Civilization
     ): MapUnit? {
-        val unit = gameInfo.ruleset.units[unitName]!!.getMapUnit(civInfo)
+        val unit = baseUnit.getMapUnit(civInfo)
 
         fun getPassableNeighbours(tile: Tile): Set<Tile> =
                 tile.neighbors.filter { unit.movement.canPassThrough(it) }.toSet()
@@ -505,7 +529,7 @@ class TileMap : IsPartOfGameInfoSerialization {
             var potentialCandidates = getPassableNeighbours(currentTile)
             while (unitToPlaceTile == null && tryCount++ < 10) {
                 unitToPlaceTile = potentialCandidates
-                        .sortedByDescending { if (unit.baseUnit.isLandUnit()) it.isLand else true } // Land units should prefer to go into land tiles
+                        .sortedByDescending { if (unit.baseUnit.isLandUnit() && !unit.cache.canMoveOnWater) it.isLand else true } // Land units should prefer to go into land tiles
                         .firstOrNull { unit.movement.canMoveTo(it) }
                 if (unitToPlaceTile != null) continue
                 // if it's not found yet, let's check their neighbours
@@ -538,7 +562,7 @@ class TileMap : IsPartOfGameInfoSerialization {
         // And update civ stats, since the new unit changes both unit upkeep and resource consumption
         civInfo.updateStatsForNextTurn()
 
-        if (unit.baseUnit.getResourceRequirements().isNotEmpty())
+        if (unit.baseUnit.getResourceRequirementsPerTurn().isNotEmpty())
             civInfo.cache.updateCivResources()
 
         return unit
@@ -607,7 +631,7 @@ class TileMap : IsPartOfGameInfoSerialization {
             .map { it to StartingLocation(it.position, it.improvement!!.removePrefix(startingLocationPrefix)) }
             .sortedBy { it.second.nation }  // vanity, or to make diffs between un-gzipped map files easier
             .forEach { (tile, startingLocation) ->
-                tile.changeImprovement(null)
+                tile.removeImprovement()
                 startingLocations.add(startingLocation)
             }
         setStartingLocationsTransients()
