@@ -179,6 +179,8 @@ open class Tile : IsPartOfGameInfoSerialization {
         toReturn.continent = continent
         toReturn.exploredBy = exploredBy
         toReturn.history = history.clone()
+        // Setting even though it's transient - where it's needed, it's a real performance saver
+        toReturn.tileResourceCache = tileResourceCache
         return toReturn
     }
 
@@ -673,7 +675,11 @@ open class Tile : IsPartOfGameInfoSerialization {
     }
 
     @delegate:Transient
-    private val isAdjacentToRiverLazy by lazy { neighbors.any { isConnectedByRiver(it) } }
+    private val isAdjacentToRiverLazy by lazy {
+        // These are so if you add a river at the bottom of the map (no neighboring tile to be connected to)
+        //   that tile is still considered adjacent to river
+        hasBottomLeftRiver || hasBottomRiver || hasBottomRightRiver
+        || neighbors.any { isConnectedByRiver(it) } }
     fun isAdjacentToRiver() = isAdjacentToRiverLazy
 
     /**
