@@ -9,13 +9,13 @@ import com.unciv.logic.multiplayer.FriendList
 import com.unciv.models.UncivSound
 import com.unciv.ui.components.FontFamilyData
 import com.unciv.ui.components.Fonts
-import com.unciv.ui.components.KeyboardBindings
+import com.unciv.ui.components.input.KeyboardBindings
 import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
 import com.unciv.utils.Display
 import com.unciv.utils.ScreenOrientation
 import java.text.Collator
 import java.time.Duration
-import java.util.*
+import java.util.Locale
 import kotlin.reflect.KClass
 import kotlin.reflect.KMutableProperty0
 
@@ -44,6 +44,7 @@ class GameSettings {
     var showResourcesAndImprovements: Boolean = true
     var showTileYields: Boolean = false
     var showUnitMovements: Boolean = false
+    var showSettlersSuggestedCityLocations: Boolean = true
 
     var checkForDueUnits: Boolean = true
     var autoUnitCycle: Boolean = true
@@ -70,6 +71,8 @@ class GameSettings {
     var autoBuildingRoads: Boolean = true
     var automatedWorkersReplaceImprovements = true
     var automatedUnitsMoveOnTurnStart: Boolean = false
+    var automatedUnitsCanUpgrade: Boolean = false
+    var automatedUnitsChoosePromotions: Boolean = false
 
     var showMinimap: Boolean = true
     var minimapSize: Int = 6    // default corresponds to 15% screen space
@@ -77,7 +80,6 @@ class GameSettings {
     val showPixelUnits: Boolean get() = unitSet != null
     var showPixelImprovements: Boolean = true
     var continuousRendering = false
-    var experimentalRendering = false
     var orderTradeOffersByAmount = true
     var confirmNextTurn = false
     var windowState = WindowState()
@@ -118,6 +120,13 @@ class GameSettings {
     /** NotificationScroll on Word Screen visibility control - mapped to NotificationsScroll.UserSetting enum */
     var notificationScroll: String = ""
 
+    /** If on, selected notifications are drawn enlarged with wider padding */
+    var enlargeSelectedNotification = true
+
+    /** Whether the Nation Picker shows icons only or the horizontal "civBlocks" with leader/nation name */
+    enum class NationPickerListMode { Icons, List }
+    var nationPickerListMode = NationPickerListMode.List
+
     /** used to migrate from older versions of the settings */
     var version: Int? = null
 
@@ -151,7 +160,7 @@ class GameSettings {
         locale = try {
             val code = LocaleCode.valueOf(languageName)
             Locale(code.language, code.country)
-        } catch (e: Exception) {
+        } catch (_: Exception) {
             Locale.getDefault()
         }
     }
