@@ -424,7 +424,7 @@ class DiplomacyScreen(
         if (otherCiv.cities.isEmpty()) return null
         val improvableResourceTiles = getImprovableResourceTiles(otherCiv)
         val improvements =
-            otherCiv.gameInfo.ruleset.tileImprovements.filter { it.value.turnsToBuild != 0 }
+            otherCiv.gameInfo.ruleset.tileImprovements.filter { it.value.turnsToBuild != -1 }
         var needsImprovements = false
 
         for (improvableTile in improvableResourceTiles)
@@ -658,10 +658,6 @@ class DiplomacyScreen(
             if (!diplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship))
                 diplomacyTable.add(getDeclareFriendshipButton(otherCiv)).row()
 
-
-            if (viewingCiv.diplomacyFunctions.canSignResearchAgreementsWith(otherCiv))
-                diplomacyTable.add(getResearchAgreementButton(otherCiv)).row()
-
             if (!diplomacyManager.hasFlag(DiplomacyFlags.Denunciation)
                     && !diplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship)
             ) diplomacyTable.add(getDenounceButton(otherCiv, diplomacyManager)).row()
@@ -744,30 +740,7 @@ class DiplomacyScreen(
         if (isNotPlayersTurn()) denounceButton.disable()
         return denounceButton
     }
-
-    private fun getResearchAgreementButton(otherCiv: Civilization): TextButton {
-        val researchAgreementButton = "Research Agreement".toTextButton()
-
-        val requiredGold = viewingCiv.diplomacyFunctions.getResearchAgreementCost()
-        researchAgreementButton.onClick {
-            val tradeTable = setTrade(otherCiv)
-            val researchAgreement =
-                    TradeOffer(Constants.researchAgreement, TradeType.Treaty, requiredGold)
-            val goldCostOfSignResearchAgreement =
-                    TradeOffer("Gold".tr(), TradeType.Gold, -requiredGold)
-            tradeTable.tradeLogic.currentTrade.theirOffers.add(researchAgreement)
-            tradeTable.tradeLogic.ourAvailableOffers.add(researchAgreement)
-            tradeTable.tradeLogic.ourAvailableOffers.add(goldCostOfSignResearchAgreement)
-            tradeTable.tradeLogic.currentTrade.ourOffers.add(researchAgreement)
-            tradeTable.tradeLogic.theirAvailableOffers.add(researchAgreement)
-            tradeTable.tradeLogic.theirAvailableOffers.add(goldCostOfSignResearchAgreement)
-            tradeTable.offerColumnsTable.update()
-            tradeTable.enableOfferButton(true)
-        }
-        if (isNotPlayersTurn()) researchAgreementButton.disable()
-        return researchAgreementButton
-    }
-
+    
     private fun getDeclareFriendshipButton(otherCiv: Civilization): TextButton {
         val declareFriendshipButton =
                 "Offer Declaration of Friendship ([30] turns)".toTextButton()
@@ -791,8 +764,6 @@ class DiplomacyScreen(
         val tradeButton = "Trade".toTextButton()
         tradeButton.onClick {
             setTrade(otherCiv).apply {
-                tradeLogic.ourAvailableOffers.apply { remove(firstOrNull { it.type == TradeType.Treaty }) }
-                tradeLogic.theirAvailableOffers.apply { remove(firstOrNull { it.type == TradeType.Treaty }) }
                 offerColumnsTable.update()
             }
         }
