@@ -7,16 +7,17 @@ import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.stats.Stats
 import com.unciv.testing.GdxTestRunner
-import com.unciv.ui.worldscreen.unit.actions.UnitActionsPillage
 import org.junit.Assert
 import org.junit.Before
 import org.junit.Test
 import org.junit.runner.RunWith
+import kotlin.math.abs
 
 @RunWith(GdxTestRunner::class)
 class GlobalUniquesTests {
 
     private lateinit var game: TestGame
+    private val epsilon = 0.01 // for float comparisons
 
     @Before
     fun initTheWorld() {
@@ -29,12 +30,12 @@ class GlobalUniquesTests {
     fun statsOnBuilding() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true)
-        val buildingName = game.createBuilding("[+1 Food]").name
+        val city = game.addCity(civInfo, tile, true)
+        val building = game.createBuilding("[+1 Food]")
 
-        cityInfo.cityConstructions.addBuilding(buildingName)
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.equals(Stats(food=1f)))
+        city.cityConstructions.addBuilding(building)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.equals(Stats(food=1f)))
     }
 
 
@@ -57,70 +58,70 @@ class GlobalUniquesTests {
     fun statsPerCity() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true)
-        val buildingName = game.createBuilding("[+1 Production] [in this city]").name
+        val city = game.addCity(civInfo, tile, true)
+        val building = game.createBuilding("[+1 Production] [in this city]")
 
-        cityInfo.cityConstructions.addBuilding(buildingName)
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.equals(Stats(production=1f)))
+        city.cityConstructions.addBuilding(building)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.equals(Stats(production=1f)))
     }
 
     @Test
     fun statsPerSpecialist() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true, initialPopulation = 2)
+        val city = game.addCity(civInfo, tile, true, initialPopulation = 2)
         val building = game.createBuilding("[+3 Gold] from every specialist [in this city]")
         val specialistName = game.createSpecialist()
         building.specialistSlots.add(specialistName, 2)
-        cityInfo.population.specialistAllocations[specialistName] = 2
+        city.population.specialistAllocations[specialistName] = 2
 
-        cityInfo.cityConstructions.addBuilding(building.name)
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Specialists"]!!.equals(Stats(gold=6f)))
+        city.cityConstructions.addBuilding(building)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Specialists"]!!.equals(Stats(gold=6f)))
     }
 
     @Test
     fun statsPerPopulation() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true, initialPopulation = 4)
+        val city = game.addCity(civInfo, tile, true, initialPopulation = 4)
         val building = game.createBuilding("[+3 Gold] per [2] population [in this city]")
 
-        cityInfo.cityConstructions.addBuilding(building.name)
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.gold == 6f)
+        city.cityConstructions.addBuilding(building)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 6f)
     }
 
     @Test
     fun statsPerXPopulation() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true, initialPopulation = 2)
+        val city = game.addCity(civInfo, tile, true, initialPopulation = 2)
         val building = game.createBuilding("[+3 Gold] <in cities with at least [3] [Population]>")
 
-        cityInfo.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.gold == 0f)
-        cityInfo.population.setPopulation(5)
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.gold == 3f)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 0f)
+        city.population.setPopulation(5)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 3f)
     }
 
     @Test
     fun statsFromCitiesOnSpecificTiles() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, tile, true)
         val building = game.createBuilding("[+3 Gold] in cities on [${Constants.desert}] tiles")
-        cityInfo.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.gold == 3f)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 3f)
         tile.baseTerrain = Constants.grassland
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.gold == 0f)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 0f)
     }
 
     @Test
@@ -128,12 +129,12 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(2)
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, tile, true)
         val building = game.createBuilding("[+4 Gold] from [${Constants.grassland}] tiles [in all cities]")
-        cityInfo.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
-        Assert.assertTrue(tile2.stats.getTileStats(cityInfo, civInfo).gold == 4f)
+        Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
     }
 
     @Test
@@ -141,17 +142,17 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, tile, true)
         val building = game.createBuilding("[+4 Gold] from [${Constants.grassland}] tiles without [${Constants.forest}] [in this city]")
-        cityInfo.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
-        game.addTileToCity(cityInfo, tile2)
-        Assert.assertTrue(tile2.stats.getTileStats(cityInfo, civInfo).gold == 4f)
+        game.addTileToCity(city, tile2)
+        Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
 
         val tile3 = game.setTileFeatures(Vector2(0f, 2f), Constants.grassland, listOf(Constants.forest))
-        game.addTileToCity(cityInfo, tile3)
-        Assert.assertFalse(tile3.stats.getTileStats(cityInfo, civInfo).gold == 4f)
+        game.addTileToCity(city, tile3)
+        Assert.assertFalse(tile3.stats.getTileStats(city, civInfo).gold == 4f)
     }
 
     @Test
@@ -159,32 +160,32 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true, initialPopulation = 2)
+        val city = game.addCity(civInfo, tile, true, initialPopulation = 2)
         val specialist = game.createSpecialist()
         val building = game.createBuilding("[+3 Faith] from every [${specialist}]")
 
-        cityInfo.cityConstructions.addBuilding(building.name)
-        cityInfo.population.specialistAllocations[specialist] = 2
+        city.cityConstructions.addBuilding(building)
+        city.population.specialistAllocations[specialist] = 2
 
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Specialists"]!!.faith == 6f)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Specialists"]!!.faith == 6f)
 
-        cityInfo.cityConstructions.removeBuilding(building.name)
+        city.cityConstructions.removeBuilding(building)
         val building2 = game.createBuilding("[+3 Faith] from every [${Constants.grassland}]")
-        cityInfo.cityConstructions.addBuilding(building2.name)
+        city.cityConstructions.addBuilding(building2)
 
         val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
-        Assert.assertTrue(tile2.stats.getTileStats(cityInfo, civInfo).faith == 3f)
+        Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 3f)
 
-        cityInfo.cityConstructions.removeBuilding(building2.name)
+        city.cityConstructions.removeBuilding(building2)
 
         val emptyBuilding = game.createBuilding()
 
         val building3 = game.createBuilding("[+3 Faith] from every [${emptyBuilding.name}]")
-        cityInfo.cityConstructions.addBuilding(emptyBuilding.name)
-        cityInfo.cityConstructions.addBuilding(building3.name)
-        cityInfo.cityStats.update()
-        Assert.assertTrue(cityInfo.cityStats.finalStatList["Buildings"]!!.faith == 3f)
+        city.cityConstructions.addBuilding(emptyBuilding)
+        city.cityConstructions.addBuilding(building3)
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.faith == 3f)
     }
 
     @Test
@@ -282,7 +283,7 @@ class GlobalUniquesTests {
         val tile = game.getTile(Vector2(0f, 0f))
         val city = game.addCity(civ, tile, true)
         val building = game.createBuilding("[+10 Science]", "[+200]% [Science]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.science == 30f)
@@ -294,7 +295,7 @@ class GlobalUniquesTests {
         val tile = game.getTile(Vector2(0f, 0f))
         val city = game.addCity(civ, tile, true)
         val building = game.createBuilding("[+10 Science]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.science == 30f)
@@ -313,13 +314,13 @@ class GlobalUniquesTests {
         val city = game.addCity(civInfo, tile, true)
         val faithBuilding = game.createBuilding()
         faithBuilding.faith = 3f
-        city.cityConstructions.addBuilding(faithBuilding.name)
+        city.cityConstructions.addBuilding(faithBuilding)
 
         val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
         tile2.changeImprovement("Farm")
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 9f)
 
-        city.cityConstructions.addBuilding(emptyBuilding.name)
+        city.cityConstructions.addBuilding(emptyBuilding)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.faith == 9f)
@@ -338,13 +339,13 @@ class GlobalUniquesTests {
         val city = game.addCity(civInfo, tile, true)
         val faithBuilding = game.createBuilding()
         faithBuilding.faith = 3f
-        city.cityConstructions.addBuilding(faithBuilding.name)
+        city.cityConstructions.addBuilding(faithBuilding)
 
         val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
         tile2.changeImprovement("Farm")
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 9f)
 
-        city.cityConstructions.addBuilding(emptyBuilding.name)
+        city.cityConstructions.addBuilding(emptyBuilding)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.faith == 9f)
@@ -390,7 +391,7 @@ class GlobalUniquesTests {
         Assert.assertTrue(city.cityStats.finalStatList[Constants.cityStates]!!.food == 3f)
 
         val building = game.createBuilding("[+100]% [Food] from City-States")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList[Constants.cityStates]!!.food == 6f)
     }
@@ -430,7 +431,7 @@ class GlobalUniquesTests {
         val city = game.addCity(civInfo, tile, true, 1)
 
         val building = game.createBuilding("Nullifies [Faith] [in this city]", "[+10 Gold, +10 Faith] [in this city]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.gold }.sum() >= 10f)
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.faith }.sum() == 0f)
@@ -444,7 +445,7 @@ class GlobalUniquesTests {
         val city = game.addCity(civInfo, tile, true, 1)
 
         val building = game.createBuilding("Nullifies Growth [in this city]", "[+10 Food, +10 Gold] [in this city]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.gold }.sum() >= 10f)
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.food }.sum() == 0f)
@@ -466,7 +467,7 @@ class GlobalUniquesTests {
 
         val buildingToConstruct = game.createBuilding()
         val building = game.createBuilding("[+300]% Production when constructing [${buildingToConstruct.name}] buildings [in all cities]", "[+1 Production]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityConstructions.addToQueue(buildingToConstruct.name)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.statPercentBonusTree.totalStats.production == 300f)
@@ -480,7 +481,7 @@ class GlobalUniquesTests {
 
         val unitToConstruct = game.createBaseUnit()
         val building = game.createBuilding("[+300]% Production when constructing [${unitToConstruct.name}] units [in all cities]", "[+1 Production]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityConstructions.addToQueue(unitToConstruct.name)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.statPercentBonusTree.totalStats.production == 300f)
@@ -494,7 +495,7 @@ class GlobalUniquesTests {
 
         val buildingToConstruct = game.createBuilding()
         val building = game.createBuilding("[+300]% Production when constructing [${buildingToConstruct.name}] wonders [in all cities]", "[+1 Production]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityConstructions.addToQueue(buildingToConstruct.name)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.statPercentBonusTree.totalStats.production == 0f)
@@ -517,48 +518,121 @@ class GlobalUniquesTests {
         city2.cityStats.update()
         Assert.assertTrue(city2.cityStats.statPercentBonusTree.totalStats.production == 0f)
 
-        city.cityConstructions.addBuilding(buildingToConstruct.name)
+        city.cityConstructions.addBuilding(buildingToConstruct)
         city2.cityStats.update()
         Assert.assertTrue(city2.cityStats.statPercentBonusTree.totalStats.production == 300f)
     }
 
     //endregion
 
+    // region Other Global Uniques
+
+    // region growth
+
+    @Test
+    fun growthPercentBonusTest() {
+        val civInfo = game.addCiv()
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val city = game.addCity(civInfo, tile, true)
+        val building = game.createBuilding("[+100]% growth [in all cities]")
+        city.cityConstructions.addBuilding(building)
+
+        city.cityStats.update()
+        Assert.assertTrue(city.cityStats.finalStatList["[Buildings] ([Growth])"]!!.equals(Stats(food=2f)))
+    }
+
+    @Test
+    fun carryOverFoodTest() {
+        val civInfo = game.addCiv("[50]% Food is carried over after population increases [in all cities]")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val city = game.addCity(civInfo, tile, true)
+
+        val foodNecessary = city.population.getFoodToNextPopulation()
+        city.population.nextTurn(foodNecessary)
+
+        Assert.assertTrue(city.population.foodStored == (foodNecessary * 0.5f).toInt())
+    }
+
+
+    @Test
+    fun foodConsumptionBySpecialistsTest() {
+        val civInfo = game.addCiv("[-50]% Food consumption by specialists [in all cities]")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val city = game.addCity(civInfo, tile, true, initialPopulation = 1)
+
+        val building = game.createBuilding()
+        val specialistName = game.createSpecialist()
+        building.specialistSlots.add(specialistName, 1)
+        city.population.specialistAllocations[specialistName] = 1
+
+        city.cityStats.update()
+        print(city.cityStats.finalStatList)
+        Assert.assertTrue(city.cityStats.finalStatList["Population"]!!.food == -1f)
+    }
+
+    // endregion growth
+
+    // region happiness
+
+    @Test
+    fun unhappinessFromCitiesPercentageTest() {
+        val civInfo = game.addCiv("[+100]% unhappiness from the number of cities")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val city = game.addCity(civInfo, tile, true)
+
+        city.cityStats.update()
+        // Because of some weird design choices, -3.6 is correct, though I would expect it to be -6 instead.
+        // As I'm not certain enough in my feeling that it should be -6, I'm changing the test to fit the code instead of the other way around.
+        // I've also written some text about this in CityStats.updateCityHappiness(). ~xlenstra
+        println(city.cityStats.happinessList)
+        Assert.assertTrue(abs(city.cityStats.happinessList["Cities"]!! - -3.6f) < epsilon) // Float rounding errors
+    }
+
+    @Test
+    fun unhappinessFromPopulationTypePercentageChangeTest() {
+        val civInfo = game.addCiv("[-50]% Unhappiness from [Population] [in all cities]")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val city = game.addCity(civInfo, tile, true, initialPopulation = 4)
+
+        city.cityStats.update()
+        // This test suffers from the same problems as `unhappinessFromCitiesPercentageTest()`.
+        // This should be -2, I believe, as every pop should add -1 happiness and 4 pop with -50% unhappiness = -2 unhappiness.
+        println(city.cityStats.happinessList)
+        Assert.assertTrue(abs(city.cityStats.happinessList["Population"]!! - -1.2f) < epsilon)
+
+        val building = game.createBuilding("[-50]% Unhappiness from [Specialists] [in all cities]")
+        val specialist = game.createSpecialist()
+        building.specialistSlots[specialist] = 2
+        city.population.specialistAllocations[specialist] = 2
+        city.cityConstructions.addBuilding(building)
+
+        city.cityStats.update()
+        println(city.cityStats.happinessList)
+        // This test suffers from the same problems as above. It should be -1, I believe.
+        Assert.assertTrue(abs(city.cityStats.happinessList["Population"]!! - -0.6f) < epsilon)
+    }
+
+
+    // endregion happiness
+
+
+    // region Great Persons
     @Test
     fun statsSpendingGreatPeople() {
         val civInfo = game.addCiv()
         val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val cityInfo = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, tile, true)
         val unit = game.addUnit("Great Engineer", civInfo, tile)
         val building = game.createBuilding("[+250 Gold] whenever a Great Person is expended")
-        cityInfo.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         civInfo.addGold(-civInfo.gold) // reset gold just to be sure
 
         unit.consume()
         Assert.assertTrue(civInfo.gold == 250)
     }
+    // endregion
 
-    @Test
-    fun pillageYieldTest() {
-        game.makeHexagonalMap(2)
-        val civInfo = game.addCiv()
-
-        val tile = game.setTileFeatures(Vector2(0f, 0f), Constants.grassland)
-        val cityTile = game.setTileFeatures(Vector2(2f,0f), Constants.grassland)
-        val cityInfo = game.addCity(civInfo, cityTile, true)
-        cityInfo.population.foodStored = 0 // just to be sure
-        civInfo.addGold(-civInfo.gold) // reset gold just to be sure
-
-        val testImprovement = game.createTileImprovement("Pillaging this improvement yields [+20 Gold, +11 Food]")
-        tile.changeImprovement(testImprovement.name)
-        val unit = game.addUnit("Warrior", civInfo, tile)
-        unit.currentMovement = 2f
-
-        val pillageAction = UnitActionsPillage.getPillageAction(unit)
-        pillageAction?.action?.invoke()
-        Assert.assertTrue("Pillaging should transfer gold to the civ", civInfo.gold == 20)
-        Assert.assertTrue("Pillaging should transfer food to the nearest city", cityInfo.population.foodStored == 11)
-    }
+    // endregion
 
 }
