@@ -28,6 +28,7 @@ import com.unciv.models.ruleset.nation.Nation
 import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
+import com.unciv.models.ruleset.unit.Promotion
 import com.unciv.models.ruleset.unit.UnitType
 
 class TestGame {
@@ -146,7 +147,7 @@ class TestGame {
             // Add a capital indicator without any other stats
             val palaceWithoutStats = createBuilding(UniqueType.IndicatesCapital.text)
             city.cityConstructions.removeBuilding("Palace")
-            city.cityConstructions.addBuilding(palaceWithoutStats.name)
+            city.cityConstructions.addBuilding(palaceWithoutStats)
         }
         return city
     }
@@ -207,4 +208,6 @@ class TestGame {
         createRulesetObject(ruleset.tileImprovements, *uniques) { TileImprovement() }
     fun createUnitType(vararg uniques: String) =
         createRulesetObject(ruleset.unitTypes, *uniques) { UnitType() }
+    fun createUnitPromotion(vararg uniques: String) =
+        createRulesetObject(ruleset.unitPromotions, *uniques) { Promotion() }
 }
